/*
 * color.c
 *
 * Copyright (c) 1995, John D. Kilburg (john@cs.unlv.edu)
 *
 * See copyright.h for more information.
 *
 */
#include "copyright.h"

#include <stdio.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <X11/IntrinsicP.h>

#include "common.h"
#include "WWWP.h"

void
SnarfPixels(rw)
WWWWidget rw;
{
  int i;
  Pixel *px;
  Visual *v;

  rw->www.ucolors = 0;
  rw->www.ncolors = 0;

  v = DefaultVisual(rw->www.dpy, rw->www.screen);
  if (rw->www.maxColors > v->map_entries) rw->www.maxColors = v->map_entries;

  if (rw->www.maxColors < 16) return;

  px = (Pixel *)alloc_mem(sizeof(Pixel) * rw->www.maxColors);

  for (i = rw->www.maxColors; i >= 16; i--)
  {
    if (XAllocColorCells(rw->www.dpy, rw->www.cmap,
			 False, NULL, 0,
			 px, i))
    {
      rw->www.ncolors = i;
      rw->www.xc = (XcmsColor *)alloc_mem(sizeof(XcmsColor) * i);
      for (i = 0; i < rw->www.ncolors; i++)
      {
	rw->www.xc[i].pixel = px[i];
	rw->www.xc[i].format = XcmsRGBFormat;
      }

      free_mem(px);

      return;
    }
  }       

  free_mem(px);

  return;
}

static int
color_compare(c1, c2)
XcmsColor *c1, *c2;
{
  if (c1->spec.RGB.red < c2->spec.RGB.red) return(-1);
  else if (c1->spec.RGB.red == c2->spec.RGB.red)
  {
    if (c1->spec.RGB.green < c2->spec.RGB.green) return(-1);
    else if (c1->spec.RGB.green == c2->spec.RGB.green)
    {
      if (c1->spec.RGB.blue < c2->spec.RGB.blue) return(-1);
      else if (c1->spec.RGB.blue == c2->spec.RGB.blue) return(0);
    }
  }

  return(1);
}

static void
InsertColor(rw, i, n)
WWWWidget rw;
int i;
XcmsColor *n;
{
  int j;
  XcmsColor *xc = rw->www.xc;
  Pixel p;

  p = xc[rw->www.ucolors].pixel;

  for (j = rw->www.ucolors; j > i; j--)
  {
    xc[j].spec.RGB.red = xc[j - 1].spec.RGB.red;
    xc[j].spec.RGB.green = xc[j - 1].spec.RGB.green;
    xc[j].spec.RGB.blue = xc[j - 1].spec.RGB.blue;
    xc[j].pixel = xc[j - 1].pixel;
  }

  xc[i].spec.RGB.red = n->spec.RGB.red;
  xc[i].spec.RGB.green = n->spec.RGB.green;
  xc[i].spec.RGB.blue = n->spec.RGB.blue;
  xc[i].pixel = p;

  rw->www.ucolors++;

  return;
}

Pixel
GetPixel(rw, r, g, b)
WWWWidget rw;
unsigned short r, g, b;
{
  Pixel p;
  int i;
  XcmsColor *xc;
  XcmsColor m;
  XcmsColor *x;

  if (rw->www.xc == NULL) return(BlackPixel(rw->www.dpy, rw->www.screen));

  xc = rw->www.xc;

  m.spec.RGB.red = r;
  m.spec.RGB.green = g;
  m.spec.RGB.blue = b;

  if ((x = (XcmsColor *)bsearch(&m, xc, rw->www.ucolors,
                                sizeof(XcmsColor), color_compare)) != NULL)
  {
    return(x->pixel);
  }

  if (rw->www.ucolors == rw->www.ncolors)
  {
    p = BlackPixel(rw->www.dpy, rw->www.screen);
  }
  else
  {
    for (i = 0; i < rw->www.ucolors; i++)
    {
      if (color_compare(&m, &xc[i]) <= 0)
      {
	InsertColor(rw, i, &m);
	break;
      }
    }
   
    if (i == rw->www.ucolors) InsertColor(rw, i, &m);

    p = xc[i].pixel;

    XcmsStoreColor(rw->www.dpy, rw->www.cmap, &xc[i]);
  }

  return(p);
}

void
FreePixels(rw)
WWWWidget rw;
{
  rw->www.ucolors = 0;

  return;
}





