/* new.c:
 *
 * random changes by john@isri.unlv.edu
 *
 * functions to allocate and deallocate structures and structure data
 *
 * jim frost 09.29.89
 *
 * Copyright 1989, 1991 Jim Frost.
 * See included file "copyright.h" for complete copyright information.
 */
#include <stdio.h>

#include "common.h"
#include "copyright.h"
#include "xloadimage.h"

/* this table is useful for quick conversions between depth and ncolors
 */

unsigned long DepthToColorsTable[] =
{
  /*  0 */ 1,
  /*  1 */ 2,
  /*  2 */ 4,
  /*  3 */ 8,
  /*  4 */ 16,
  /*  5 */ 32,
  /*  6 */ 64,
  /*  7 */ 128,
  /*  8 */ 256,
  /*  9 */ 512,
  /* 10 */ 1024,
  /* 11 */ 2048,
  /* 12 */ 4096,
  /* 13 */ 8192,
  /* 14 */ 16384,
  /* 15 */ 32768,
  /* 16 */ 65536,
  /* 17 */ 131072,
  /* 18 */ 262144,
  /* 19 */ 524288,
  /* 20 */ 1048576,
  /* 21 */ 2097152,
  /* 22 */ 4194304,
  /* 23 */ 8388608,
  /* 24 */ 16777216,
  /* 25 */ 33554432,
  /* 26 */ 67108864,
  /* 27 */ 134217728,
  /* 28 */ 268435456,
  /* 29 */ 536870912,
  /* 30 */ 1073741824,
  /* 31 */ 2147483648,
  /* 32 */ 2147483648 /* bigger than unsigned int; this is good enough */
};

unsigned long
colorsToDepth(ncolors)
unsigned long ncolors;
{
  unsigned long a;

  for (a = 0; (a < 32) && (DepthToColorsTable[a] < ncolors); a++)
    /* EMPTY */
    ;
  return(a);
}

Image *
newBitImage(width, height)
unsigned int width, height;
{
  Image        *image;

  image = (Image *)alloc_mem(sizeof(Image));
  image->type = IBITMAP;
  image->width = width;
  image->height = height;
  image->depth = 1;
  image->data = (byte *)alloc_mem(sizeof(byte) * width * height);
  memset(image->data, 0, sizeof(byte) * width * height);

  return(image);
}

Image *
newRGBImage(width, height, depth)
unsigned int width, height, depth;
{
  Image *image;
  unsigned int pixlen, numcolors;

  pixlen = (depth / 8) + (depth % 8 ? 1 : 0);
  if (pixlen == 0) pixlen = 1;
  numcolors = depthToColors(depth);

  image = (Image *)alloc_mem(sizeof(Image));
  image->type = IRGB;
  image->width = width;
  image->height = height;
  image->depth = depth;
  image->pixlen = pixlen;
  image->data = (byte *)alloc_mem(width * height * pixlen);

  image->rgb.used = 0;
  image->rgb.compressed = 0;
  image->rgb.size = 2;

  memset(image->pc, 0, 32768 * sizeof(image->pc[0]));

  return(image);
}

void
freeImage(image)
Image *image;
{
  if (image->data != NULL) free_mem((char *)image->data);
  free_mem((char *)image);

  return;
}
